package betterquesting.api2.client.gui.panels.lists;

import betterquesting.api2.client.gui.misc.IGuiRect;
import com.google.common.base.Stopwatch;
import java.util.ArrayDeque;
import java.util.Iterator;
import java.util.concurrent.TimeUnit;

/* loaded from: betterquesting_api2_client_gui_panels_lists.jar:betterquesting/api2/client/gui/panels/lists/CanvasSearch.class */
public abstract class CanvasSearch<T, E> extends CanvasScrolling {
    private String searchTerm;
    private Iterator<E> searching;
    private final Stopwatch searchTime;
    private int resultWidth;
    private int searchIdx;
    private final ArrayDeque<T> pendingResults;

    protected abstract Iterator<E> getIterator();

    protected abstract void queryMatches(E e, String str, ArrayDeque<T> arrayDeque);

    protected abstract boolean addResult(T t, int i, int i2);

    public CanvasSearch(IGuiRect rect) {
        super(rect);
        this.searchTerm = "";
        this.searching = null;
        this.searchTime = Stopwatch.createStarted();
        this.resultWidth = 256;
        this.searchIdx = 0;
        this.pendingResults = new ArrayDeque<>();
    }

    public void setSearchFilter(String text) {
        this.searchTerm = text.toLowerCase();
        refreshSearch();
    }

    @Override // betterquesting.api2.client.gui.panels.lists.CanvasScrolling
    public void initPanel() {
        super.initPanel();
        refreshSearch();
    }

    @Override // betterquesting.api2.client.gui.panels.lists.CanvasScrolling
    public void drawPanel(int mx, int my, float partialTick) {
        updateSearch();
        updateResults();
        super.drawPanel(mx, my, partialTick);
    }

    public void refreshSearch() {
        resetCanvas();
        this.searchIdx = 0;
        this.searching = getIterator();
        this.resultWidth = getTransform().getWidth();
        this.pendingResults.clear();
    }

    private void updateSearch() {
        if (this.searching == null) {
            return;
        }
        if (!this.searching.hasNext()) {
            this.searching = null;
            return;
        }
        this.searchTime.reset().start();
        while (this.searching.hasNext() && this.searchTime.elapsed(TimeUnit.MILLISECONDS) < 10) {
            E entry = this.searching.next();
            if (entry != null) {
                queryMatches(entry, this.searchTerm, this.pendingResults);
            }
        }
        this.searchTime.stop();
    }

    private void updateResults() {
        if (this.pendingResults.isEmpty()) {
            return;
        }
        this.searchTime.reset().start();
        while (!this.pendingResults.isEmpty() && this.searchTime.elapsed(TimeUnit.MILLISECONDS) < 100) {
            if (addResult(this.pendingResults.poll(), this.searchIdx, this.resultWidth)) {
                this.searchIdx++;
            }
        }
        this.searchTime.stop();
    }
}
